#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2021 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Luis Cañas-Díaz <lcanas@bitergia.com>
#     Alvaro del Castillo <acs@bitergia.com>
#     Quan Zhou <quan@bitergia.com>
#

import logging
import time

from datetime import datetime, timedelta

from elasticsearch import Elasticsearch, RequestsHttpConnection

from grimoire_elk.elk import (do_studies,
                              enrich_backend,
                              refresh_projects,
                              refresh_identities,
                              retain_identities,
                              populate_identities_index)
from grimoire_elk.elastic_items import ElasticItems
from grimoire_elk.elastic import ElasticSearch
from grimoire_elk.enriched.git import GitEnrich
from grimoire_elk.utils import get_elastic
from grimoire_elk.enriched.sortinghat_gelk import SortingHat

from sirmordred.error import DataEnrichmentError
from sirmordred.task import Task
from sirmordred.task_manager import TasksManager
from sirmordred.task_projects import TaskProjects


logger = logging.getLogger(__name__)


class TaskEnrich(Task):
    """ Basic class shared by all enriching tasks """

    def __init__(self, config, backend_section=None, allowed_repos=None):
        super().__init__(config)
        self.backend_section = backend_section
        self.allowed_repos = set(allowed_repos) if allowed_repos else None
        # This will be options in next iteration
        self.clean = False
        # check whether the aliases has beed already created
        self.enrich_aliases = False
        if self.db_sh is None and self.db_host is None:
            self.db = None
        else:
            self.db = self.client

        autorefresh_interval = self.conf['es_enrichment']['autorefresh_interval']
        self.last_autorefresh = self.__update_last_autorefresh(days=autorefresh_interval)
        self.last_autorefresh_studies = self.last_autorefresh
        self.last_sortinghat_import = None

    def select_aliases(self, cfg, backend_section):

        aliases_file = cfg['general']['aliases_file']
        aliases = self.load_aliases_from_json(aliases_file)
        if backend_section in aliases:
            found = aliases[backend_section]['enrich']
        else:
            found = [self.get_backend(backend_section)]

        return found

    @staticmethod
    def __update_last_autorefresh(days=None):
        if not days:
            return datetime.utcnow()
        else:
            return datetime.utcnow() - timedelta(days=days)

    def __load_studies(self):
        studies = [study for study in self.conf[self.backend_section]['studies'] if study.strip() != ""]
        if not studies:
            logger.debug('No studies for %s' % self.backend_section)
            return None

        studies_args = []

        for study in studies:
            if study not in self.conf:
                msg = 'Missing config for study: %s' % study
                logger.error(msg)
                raise DataEnrichmentError(msg)

            study_params = self.conf[study]
            studies_args.append({"name": study,
                                 "type": study.split(":")[0],
                                 "params": study_params})

        return studies_args

    def __enrich_items(self):

        time_start = datetime.now()

        logger.info('[%s] enrichment phase starts', self.backend_section)

        cfg = self.config.get_conf()

        if 'scroll_size' in cfg['general']:
            ElasticItems.scroll_size = cfg['general']['scroll_size']

        if 'bulk_size' in cfg['general']:
            ElasticSearch.max_items_bulk = cfg['general']['bulk_size']

        no_incremental = False
        # not used due to https://github.com/chaoss/grimoirelab-elk/pull/773
        github_token = None
        pair_programming = False
        node_regex = None

        if self.backend_section in cfg and 'pair-programming' in cfg[self.backend_section]:
            pair_programming = cfg[self.backend_section]['pair-programming']
        if 'jenkins' in cfg and 'node_regex' in cfg['jenkins']:
            node_regex = cfg['jenkins']['node_regex']
        only_studies = False
        only_identities = False

        # repos could change between executions because changes in projects
        repos = TaskProjects.get_repos_by_backend_section(self.backend_section, raw=False)

        if not repos:
            logger.warning("No enrich repositories for %s", self.backend_section)
        elif self.allowed_repos is not None:
            # Filter repos to only those specified
            repos = sorted(list(set(repos) & self.allowed_repos))

        # Get the metadata__timestamp value of the last item inserted in the enriched index before
        # looping over the repos which data is stored in the same index. This is needed to make sure
        # that the incremental enrichment works for data sources that are collected globally but only
        # partially enriched.
        elastic_enrich = get_elastic(cfg['es_enrichment']['url'], cfg[self.backend_section]['enriched_index'])
        last_enrich_date = elastic_enrich.get_last_item_field("metadata__timestamp")
        if last_enrich_date:
            last_enrich_date = last_enrich_date.replace(tzinfo=None)

        for repo in repos:
            repo, repo_labels = self._extract_repo_tags(self.backend_section, repo)
            _, repo_spaces = self._extract_repo_tags(self.backend_section, repo, "spaces")
            p2o_args = self._compose_p2o_params(self.backend_section, repo)
            filter_raw = p2o_args['filter-raw'] if 'filter-raw' in p2o_args else None
            jenkins_rename_file = p2o_args['jenkins-rename-file'] if 'jenkins-rename-file' in p2o_args else None
            url = p2o_args['url']
            # Second process perceval params from repo
            backend_args = self._compose_perceval_params(self.backend_section, url)
            studies_args = None

            backend = self.get_backend(self.backend_section)
            if 'studies' in self.conf[self.backend_section] and \
                    self.conf[self.backend_section]['studies']:
                studies_args = self.__load_studies()

            logger.info('[%s] enrichment starts for %s', self.backend_section, self.anonymize_url(repo))
            es_enrich_aliases = self.select_aliases(cfg, self.backend_section)

            try:
                es_col_url = self._get_collection_url()
                enrich_backend(es_col_url, self.clean, backend, backend_args,
                               self.backend_section,
                               cfg[self.backend_section]['raw_index'],
                               cfg[self.backend_section]['enriched_index'],
                               cfg['projects']['projects_file'],
                               self.db_sh,
                               no_incremental, only_identities,
                               github_token,
                               False,  # studies are executed in its own Task
                               only_studies,
                               cfg['es_enrichment']['url'],
                               None,  # args.events_enrich
                               self.db_user,
                               self.db_password,
                               self.db_host,
                               self.db_port,
                               self.db_path,
                               self.db_ssl,
                               self.db_verify_ssl,
                               self.db_tenant,
                               None,  # args.refresh_projects,
                               None,  # args.refresh_identities,
                               author_id=None,
                               author_uuid=None,
                               filter_raw=filter_raw,
                               jenkins_rename_file=jenkins_rename_file,
                               unaffiliated_group=self.db_unaffiliate_group,
                               pair_programming=pair_programming,
                               node_regex=node_regex,
                               studies_args=studies_args,
                               es_enrich_aliases=es_enrich_aliases,
                               last_enrich_date=last_enrich_date,
                               projects_json_repo=repo,
                               repo_labels=repo_labels,
                               repo_spaces=repo_spaces)
            except Exception as ex:
                logger.error("Something went wrong producing enriched data for %s . "
                             "Using the backend_args: %s ", self.backend_section, str(backend_args))
                logger.error("Exception: %s", ex)
                raise DataEnrichmentError('Failed to produce enriched data for ' + self.backend_section)

            logger.info('[%s] enrichment finished for %s', self.backend_section, self.anonymize_url(repo))

        spent_time = str(datetime.now() - time_start).split('.')[0]
        logger.info('[%s] enrichment phase finished in %s', self.backend_section, spent_time)

    def __autorefresh(self, enrich_backend, studies=False):
        # Refresh projects
        field_id = enrich_backend.get_field_unique_id()

        if False:
            # TODO: Waiting that the project info is loaded from yaml files
            logger.info("Refreshing project field in enriched index")
            field_id = enrich_backend.get_field_unique_id()
            eitems = refresh_projects(enrich_backend)
            enrich_backend.elastic.bulk_upload(eitems, field_id)

        # Refresh identities

        if studies:
            after = self.last_autorefresh_studies
        else:
            after = self.last_autorefresh

        logger.info(f"[{self.backend_section}] Refreshing identities from {after}")

        # As we are going to recover modified identities just below, store this time
        # to make sure next iteration we are not loosing any modification, but don't
        # update corresponding field with this below until we make sure the update
        # was done in ElasticSearch
        next_autorefresh = self.__update_last_autorefresh()

        logger.debug('Getting last modified identities from SH since %s for %s', after, self.backend_section)

        author_fields = ["author_uuid"]
        for role in enrich_backend.roles:
            author_fields.append(role + '_uuid')
        try:
            meta_fields = enrich_backend.meta_fields
            author_fields += meta_fields
        except AttributeError:
            pass

        logger.debug("Refreshing identity ids for %s", self.backend_section)
        total = 0
        time_start = datetime.now()
        for individuals in SortingHat.search_last_modified_identities(self.client, after):
            eitems = refresh_identities(enrich_backend, author_fields=author_fields, individuals=individuals)
            enrich_backend.elastic.bulk_upload(eitems, field_id)
            total += len(individuals)
            logger.debug(f"[{self.backend_section}] Individuals refreshed: {total}")
        else:
            logger.debug("No ids to be refreshed found")

        spent_time = str(datetime.now() - time_start).split('.')[0]
        logger.info(f'[{self.backend_section}] Refreshed {total} identities in {spent_time}')

        # Update corresponding autorefresh date
        if studies:
            self.last_autorefresh_studies = next_autorefresh
        else:
            self.last_autorefresh = next_autorefresh

    def __autorefresh_studies(self, cfg):
        """Execute autorefresh for areas of code study if configured"""

        if 'studies' not in self.conf[self.backend_section] or \
                'enrich_areas_of_code:git' not in self.conf[self.backend_section]['studies']:
            logger.debug("Not doing autorefresh for studies, Areas of Code study is not active.")
            return

        aoc_index = self.conf['enrich_areas_of_code:git'].get('out_index', GitEnrich.GIT_AOC_ENRICHED)

        # if `out_index` exists but has no value, use default
        if not aoc_index:
            aoc_index = GitEnrich.GIT_AOC_ENRICHED

        logger.debug("Autorefresh for Areas of Code study index: %s", aoc_index)

        es = Elasticsearch([self.conf['es_enrichment']['url']], timeout=100, retry_on_timeout=True,
                           verify_certs=self._get_enrich_backend().elastic.requests.verify,
                           connection_class=RequestsHttpConnection)

        if not es.indices.exists(index=aoc_index):
            logger.debug("Not doing autorefresh, index doesn't exist for Areas of Code study")
            return

        logger.debug("Doing autorefresh for Areas of Code study")

        # Create a GitEnrich backend tweaked to work with AOC index
        aoc_backend = GitEnrich(db_sortinghat=self.db_sh, json_projects_map=cfg['projects']['projects_file'],
                                db_user=self.db_user, db_password=self.db_password, db_host=self.db_host,
                                db_port=self.db_port, db_path=self.db_path, db_ssl=self.db_ssl,
                                db_verify_ssl=self.db_verify_ssl, db_tenant=self.db_tenant)
        aoc_backend.mapping = None
        aoc_backend.roles = ['author']
        elastic_enrich = get_elastic(self.conf['es_enrichment']['url'],
                                     aoc_index, clean=False, backend=aoc_backend)
        aoc_backend.set_elastic(elastic_enrich)

        self.__autorefresh(aoc_backend, studies=True)

    def __studies(self, retention_time):
        """ Execute the studies configured for the current backend """
        log_prefix = "[" + self.backend_section + "]"
        cfg = self.config.get_conf()
        if 'studies' not in cfg[self.backend_section] or not \
           cfg[self.backend_section]['studies']:
            logger.info('%s no studies phase', log_prefix)
            return

        studies = [study for study in cfg[self.backend_section]['studies'] if study.strip() != ""]
        if not studies:
            logger.info('%s no studies phase active', log_prefix)
            return

        logger.info('%s studies phase start', log_prefix)
        time.sleep(2)  # Wait so enrichment has finished in ES
        enrich_backend = self._get_enrich_backend()
        ocean_backend = self._get_ocean_backend(enrich_backend)

        active_studies = []
        all_studies = enrich_backend.studies
        all_studies_names = [study.__name__ for study in enrich_backend.studies]

        # Time to check that configured studies are valid
        logger.debug("%s All studies: %s", log_prefix, all_studies_names)
        logger.debug("%s Configured studies %s", log_prefix, studies)
        cfg_studies_types = [study.split(":")[0] for study in studies]
        if not set(cfg_studies_types).issubset(set(all_studies_names)):
            logger.error('%s Wrong studies names: %s', log_prefix, studies)
            raise RuntimeError('Wrong studies names ', self.backend_section, studies)

        for study in enrich_backend.studies:
            if study.__name__ in cfg_studies_types:
                active_studies.append(study)

        enrich_backend.studies = active_studies
        logger.info("%s Executing studies %s" % (log_prefix, [study for study in studies]))

        studies_args = self.__load_studies()
        study_aliases = self.select_aliases(cfg, "studies_aliases")
        for study_arg in studies_args:
            alias = [study_alias['alias'] for study_alias in study_aliases if study_arg['type'] == study_alias['name']]
            if alias:
                study_arg['params']['alias'] = alias[0]

        do_studies(ocean_backend, enrich_backend, studies_args, retention_time=retention_time)
        # Return studies to its original value
        enrich_backend.studies = all_studies

        logger.info('%s studies phase end', log_prefix)

    def retain_identities(self, retention_time):
        """Retain the identities in SortingHat based on the `retention_time`
        value declared in the setup.cfg.

        :param retention_time: maximum number of minutes wrt the current date to retain the SortingHat data
        """
        enrich_es = self.conf['es_enrichment']['url']
        sortinghat_db = self.db
        sortinghat_client = self.client
        current_data_source = self.get_backend(self.backend_section)
        active_data_sources = self.config.get_active_data_sources()

        if retention_time is None:
            logger.debug("[identities retention] Retention policy disabled, no identities will be deleted.")
            return

        if retention_time <= 0:
            logger.debug("[identities retention] Retention time must be greater than 0.")
            return

        logger.info('[%s] identities retention start', self.backend_section)

        logger.info('[%s] populate identities index start', self.backend_section)
        # Upload the unique identities seen in the items to the index `grimoirelab_identities_cache`
        populate_identities_index(self.conf['es_enrichment']['url'],
                                  self.conf[self.backend_section]['enriched_index'])
        logger.info('[%s] populate identities index end', self.backend_section)

        # Delete the unique identities in SortingHat which have not been seen in
        # `grimoirelab_identities_cache` during the retention time, and delete the orphan
        # unique identities (those ones in SortingHat but not in `grimoirelab_identities_cache`)
        retain_identities(retention_time, enrich_es, sortinghat_db, current_data_source, active_data_sources)

    def execute(self):
        cfg = self.config.get_conf()

        if 'enrich' in cfg[self.backend_section] and not cfg[self.backend_section]['enrich']:
            logger.info('%s enrich disabled', self.backend_section)
            return

        # ** START SYNC LOGIC **
        # Check that identities tasks are not active before executing
        while True:
            time.sleep(10)  # check each 10s if the enrichment could start
            with TasksManager.IDENTITIES_TASKS_ON_LOCK:
                with TasksManager.NUMBER_ENRICH_TASKS_ON_LOCK:
                    in_identities = TasksManager.IDENTITIES_TASKS_ON
                    if not in_identities:
                        # The enrichment can be started
                        TasksManager.NUMBER_ENRICH_TASKS_ON += 1
                        logger.debug("Number of enrichment tasks active: %i",
                                     TasksManager.NUMBER_ENRICH_TASKS_ON)
                        break
                    else:
                        logger.debug("%s Waiting for enrich until identities is done.",
                                     self.backend_section)
        #  ** END SYNC LOGIC **

        try:
            self.__enrich_items()

            logger.info('[%s] data retention start', self.backend_section)
            retention_time = cfg['general']['retention_time']
            # Delete the items updated before a given date
            self.retain_data(retention_time,
                             self.conf['es_enrichment']['url'],
                             self.conf[self.backend_section]['enriched_index'])
            logger.info('[%s] data retention end', self.backend_section)

            if self.db:
                self.retain_identities(retention_time)
                logger.info('[%s] identities retention end', self.backend_section)

            autorefresh = cfg['es_enrichment']['autorefresh']

            if autorefresh and self.db:
                logger.info('[%s] autorefresh start', self.backend_section)
                self.__autorefresh(self._get_enrich_backend())
                logger.info('[%s] autorefresh end', self.backend_section)
            else:
                logger.info('[%s] autorefresh not active', self.backend_section)

            self.__studies(retention_time)

            if autorefresh and self.db:
                logger.info('[%s] autorefresh for studies start', self.backend_section)
                self.__autorefresh_studies(cfg)
                logger.info('[%s] autorefresh for studies end', self.backend_section)
            else:
                logger.info('[%s] autorefresh for studies not active', self.backend_section)

        except Exception as e:
            raise e
        finally:
            with TasksManager.NUMBER_ENRICH_TASKS_ON_LOCK:
                TasksManager.NUMBER_ENRICH_TASKS_ON -= 1



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2019 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Alvaro del Castillo <acs@bitergia.com>


import os
import json
import sys
import unittest


# Hack to make sure that tests import the right packages
# due to setuptools behaviour
sys.path.insert(0, '..')

from sirmordred.config import Config
from sirmordred.task import Task

CONF_FILE = 'test.cfg'
BACKEND_NAME = 'stackexchange'
COLLECTION_URL_STACKEXCHANGE = 'http://127.0.0.1:9200'
REPO_NAME = 'https://stackoverflow.com/questions/tagged/ovirt'


def read_file(filename, mode='r'):
    with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), filename), mode) as f:
        content = f.read()
    return content


class TestTask(unittest.TestCase):
    """Task tests"""

    def test_initialization(self):
        """Test whether attributes are initializated"""

        config = Config(CONF_FILE)
        task = Task(config)

        self.assertEqual(task.config, config)
        self.assertEqual(task.db_sh, task.conf['sortinghat']['database'])
        self.assertEqual(task.db_user, task.conf['sortinghat']['user'])
        self.assertEqual(task.db_password, task.conf['sortinghat']['password'])
        self.assertEqual(task.db_host, task.conf['sortinghat']['host'])

    def test_run(self):
        """Test whether the Task could be run"""

        config = Config(CONF_FILE)
        task = Task(config)
        self.assertEqual(task.execute(), None)

    def test_compose_p2o_params(self):
        """Test whether p2o params are built correctly for a backend and a repository"""

        config = Config(CONF_FILE)
        task = Task(config)
        params = task._compose_p2o_params("stackexchange", "https://stackoverflow.com/questions/tagged/example")
        self.assertDictEqual(params, {'url': "https://stackoverflow.com/questions/tagged/example"})

        params = task._compose_p2o_params("mediawiki",
                                          "https://wiki-archive.opendaylight.org "
                                          "https://wiki-archive.opendaylight.org/view")
        self.assertDictEqual(params, {'url': "https://wiki-archive.opendaylight.org "
                                             "https://wiki-archive.opendaylight.org/view"})

        params = task._compose_p2o_params("mediawiki",
                                          "https://wiki-archive.opendaylight.org "
                                          "https://wiki-archive.opendaylight.org/view "
                                          "--filter-no-collection=true")
        self.assertDictEqual(params, {'url': "https://wiki-archive.opendaylight.org "
                                             "https://wiki-archive.opendaylight.org/view",
                                      "filter-no-collection": "true"})

    def test_extract_repo_tags(self):
        """Test the extraction of tags in repositories"""

        config = Config(CONF_FILE)
        task = Task(config)
        url, tags = task._extract_repo_tags("git", "https://github.com/zhquan_example/repo --labels=[ENG, SUPP]")
        self.assertEqual(url, "https://github.com/zhquan_example/repo")
        self.assertListEqual(tags, ["ENG", "SUPP"])

        # By default it extracts '--labels'
        url, tags = task._extract_repo_tags("confluence", "https://example.com --spaces=[HOME]")
        self.assertEqual(url, "https://example.com --spaces=[HOME]")
        self.assertListEqual(tags, [])

        # To extracts '--spaces' add tag_type='spaces'
        url, tags = task._extract_repo_tags("confluence", "https://example.com --spaces=[HOME]", tag_type="spaces")
        self.assertEqual(url, "https://example.com")
        self.assertListEqual(tags, ["HOME"])

    def test_compose_perceval_params(self):
        """Test whether perceval params are built correctly for a backend and a repository"""

        expected_repo_params = json.loads(read_file('data/task-params-expected'))

        config = Config(CONF_FILE)
        task = Task(config)

        for backend in expected_repo_params.keys():
            repo = expected_repo_params.get(backend)['repo']
            perceval_params = task._compose_perceval_params(backend, repo)
            expected_params = expected_repo_params.get(backend)['params']

            self.assertEqual(expected_params.sort(), perceval_params.sort())

    def test_get_collection_url(self):
        """Test whether the collection url could be overwritten in a backend"""

        config = Config(CONF_FILE)
        task = Task(config)
        task.backend_section = "stackexchange"

        self.assertEqual(task._get_collection_url(), COLLECTION_URL_STACKEXCHANGE)


if __name__ == "__main__":
    unittest.main(warnings='ignore')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2019 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Valerio Cosentino <valcos@bitergia.com>

import sys
import threading
import unittest

# Hack to make sure that tests import the right packages
# due to setuptools behaviour
sys.path.insert(0, '..')

from sirmordred.sirmordred import SirMordred
from sirmordred.config import Config
from sirmordred.task_manager import TasksManager
from sirmordred.task_collection import TaskRawDataCollection
from sirmordred.task_enrich import TaskEnrich
from sirmordred.task_projects import TaskProjects

CONF_FILE = 'test.cfg'


class TestTasksManager(unittest.TestCase):
    """TasksManager tests"""

    def setUp(self):
        self.config = Config(CONF_FILE)
        mordred = SirMordred(self.config)

        task = TaskProjects(self.config)
        self.assertEqual(task.execute(), None)

        self.backends = mordred._get_repos_by_backend()
        self.backend_tasks = [TaskRawDataCollection, TaskEnrich]
        self.stopper = threading.Event()

    def tearDown(self):
        pass

    def test_repos_by_backend(self):
        """Test whether the repos for each backend section are properly loaded"""

        self.assertTrue(list(self.backends.keys()), 36)

        backends = list(self.backends.keys())
        backends.sort()

        backend = backends[0]
        self.assertEqual(backend, 'askbot')
        self.assertEqual(self.backends[backend], ['https://ask.puppet.com'])

        backend = backends[1]
        self.assertEqual(backend, 'bugzilla')
        self.assertEqual(self.backends[backend], ['https://bugs.eclipse.org/bugs/'])

        backend = backends[2]
        self.assertEqual(backend, 'bugzillarest')
        self.assertEqual(self.backends[backend], ['https://bugzilla.mozilla.org'])

        backend = backends[3]
        self.assertEqual(backend, 'confluence')
        self.assertEqual(self.backends[backend], ['https://wiki.open-o.org/'])

        backend = backends[4]
        self.assertEqual(backend, 'discourse')
        self.assertEqual(self.backends[backend], ['https://foro.mozilla-hispano.org/'])

        backend = backends[5]
        self.assertEqual(backend, 'dockerhub')
        self.assertEqual(self.backends[backend], ['bitergia kibiter'])

        backend = backends[6]
        self.assertEqual(backend, 'functest')
        self.assertEqual(self.backends[backend], ['http://testresults.opnfv.org/test/'])

        backend = backends[7]
        self.assertEqual(backend, 'gerrit')
        self.assertEqual(self.backends[backend], ['review.openstack.org'])

        backend = backends[8]
        self.assertEqual(backend, 'git')
        self.assertEqual(self.backends[backend],
                         ["https://github.com/VizGrimoire/GrimoireLib "
                          "--filter-raw-prefix=data.files.file:grimoirelib_alch,data.files.file:README.md",
                          "https://github.com/MetricsGrimoire/CMetrics"])

        backend = backends[9]
        self.assertEqual(backend, 'github')
        self.assertEqual(self.backends[backend], ['https://github.com/grimoirelab/perceval'])

        backend = backends[10]
        self.assertEqual(backend, 'github:pull')
        self.assertEqual(self.backends[backend], ['https://github.com/grimoirelab/perceval'])

        backend = backends[11]
        self.assertEqual(backend, 'gitlab')
        self.assertEqual(self.backends[backend], ['https://gitlab.com/inkscape/inkscape-web'])

        backend = backends[12]
        self.assertEqual(backend, 'google_hits')
        self.assertEqual(self.backends[backend], ['bitergia grimoirelab'])

        backend = backends[13]
        self.assertEqual(backend, 'hyperkitty')
        self.assertEqual(self.backends[backend],
                         ['https://lists.mailman3.org/archives/list/mailman-users@mailman3.org'])

        backend = backends[14]
        self.assertEqual(backend, 'jenkins')
        self.assertEqual(self.backends[backend], ['https://build.opnfv.org/ci'])

        backend = backends[15]
        self.assertEqual(backend, 'jira')
        self.assertEqual(self.backends[backend], ['https://jira.opnfv.org'])

        backend = backends[16]
        self.assertEqual(backend, 'mattermost')
        self.assertEqual(self.backends[backend], ['https://chat.openshift.io 8j366ft5affy3p36987pcugaoa'])

        backend = backends[17]
        self.assertEqual(backend, 'mattermost:group1')
        self.assertEqual(self.backends[backend], ['https://chat.openshift.io 8j366ft5affy3p36987cip'])

        backend = backends[18]
        self.assertEqual(backend, 'mattermost:group2')
        self.assertEqual(self.backends[backend], ['https://chat.openshift.io 8j366ft5affy3p36987ciop'])

        backend = backends[19]
        self.assertEqual(backend, 'mbox')
        self.assertEqual(self.backends[backend], ['metrics-grimoire ~/.perceval/mbox'])

        backend = backends[20]
        self.assertEqual(backend, 'mediawiki')
        self.assertEqual(self.backends[backend], ['https://wiki.mozilla.org'])

        backend = backends[21]
        self.assertEqual(backend, 'meetup')
        self.assertEqual(self.backends[backend], ['South-East-Puppet-User-Group'])

        backend = backends[22]
        self.assertEqual(backend, 'mozillaclub')
        self.assertEqual(self.backends[backend],
                         ['https://spreadsheets.google.com/feeds/cells/'
                          '1QHl2bjBhMslyFzR5XXPzMLdzzx7oeSKTbgR5PM8qp64/ohaibtm/public/values?alt=json'])

        backend = backends[23]
        self.assertEqual(backend, 'nntp')
        self.assertEqual(self.backends[backend], ['news.mozilla.org mozilla.dev.project-link'])

        backend = backends[24]
        self.assertEqual(backend, 'phabricator')
        self.assertEqual(self.backends[backend], ['https://phabricator.wikimedia.org'])

        backend = backends[25]
        self.assertEqual(backend, 'pipermail')
        self.assertEqual(self.backends[backend], ['https://mail.gnome.org/archives/libart-hackers/'])

        backend = backends[26]
        self.assertEqual(backend, 'puppetforge')
        self.assertEqual(self.backends[backend], [''])

        backend = backends[27]
        self.assertEqual(backend, 'redmine')
        self.assertEqual(self.backends[backend], ['http://tracker.ceph.com/'])

        backend = backends[28]
        self.assertEqual(backend, 'remo')
        self.assertEqual(self.backends[backend], ['https://reps.mozilla.org'])

        backend = backends[29]
        self.assertEqual(backend, 'remo:activities')
        self.assertEqual(self.backends[backend], ['https://reps.mozilla.org'])

        backend = backends[30]
        self.assertEqual(backend, 'rss')
        self.assertEqual(self.backends[backend], ['https://blog.bitergia.com/feed/'])

        backend = backends[31]
        self.assertEqual(backend, 'slack')
        self.assertEqual(self.backends[backend], ['C7LSGB0AU'])

        backend = backends[32]
        self.assertEqual(backend, 'stackexchange')
        self.assertEqual(self.backends[backend],
                         ["https://stackoverflow.com/questions/tagged/ovirt",
                          "https://stackoverflow.com/questions/tagged/rdo",
                          "https://stackoverflow.com/questions/tagged/kibana"])

        backend = backends[33]
        self.assertEqual(backend, 'supybot')
        self.assertEqual(self.backends[backend],
                         ['openshift ~/.perceval/irc/percevalbot/logs/ChannelLogger/freenode/#openshift/'])

        backend = backends[34]
        self.assertEqual(backend, 'telegram')
        self.assertEqual(self.backends[backend], ['Mozilla_analytics'])

        backend = backends[35]
        self.assertEqual(backend, 'twitter')
        self.assertEqual(self.backends[backend], ['bitergia'])

    def test_initialization(self):
        """Test whether attributes are initializated"""

        small_delay = 0
        first_backend = self.backends[list(self.backends.keys())[0]]
        manager = TasksManager(self.backend_tasks, first_backend, self.stopper, self.config, timer=small_delay)

        self.assertEqual(manager.config, self.config)
        self.assertEqual(manager.stopper, self.stopper)
        self.assertEqual(manager.tasks_cls, self.backend_tasks)
        self.assertEqual(manager.backend_section, first_backend)
        self.assertEqual(manager.timer, small_delay)
        self.assertEqual(manager.tasks, [])

    def test_add_task(self):
        """Test whether tasks are properly added"""

        small_delay = 0
        first_backend = list(self.backends.keys())[0]
        manager = TasksManager(self.backend_tasks, first_backend, self.stopper, self.config, timer=small_delay)

        self.assertEqual(manager.tasks, [])

        for tc in manager.tasks_cls:
            task = tc(manager.config)
            task.set_backend_section(manager.backend_section)
            manager.tasks.append(task)

        self.assertEqual(len(manager.tasks), len(manager.tasks_cls))

    def test_run_on_error(self):
        """Test whether an exception is thrown if a task fails"""

        small_delay = 0
        manager = TasksManager(self.backend_tasks, "fake-section", self.stopper, self.config, timer=small_delay)

        with self.assertRaises(Exception):
            manager.run()


if __name__ == "__main__":
    # logging.basicConfig(level=logging.DEBUG, format='%(asctime)s %(message)s')
    unittest.main(warnings='ignore')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2019 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Luis Cañas-Díaz <lcanas@bitergia.com>


class DataCollectionError(Exception):
    """Exception raise for error calling feed_backend method
    """
    def __init__(self, expression):
        self.expression = expression


class ElasticSearchError(Exception):
    """Exception raised for errors in the list of backends
    """
    def __init__(self, expression):
        self.expression = expression


class DataEnrichmentError(Exception):
    """Exception raised for error calling the enrich_backend method
    """
    def __init__(self, expression):
        self.expression = expression


class ConfigError(Exception):
    """Exception raised for errors in the configuration file.

    Attributes:
        expression -- input expression in which the error occurred
        message -- explanation of the error
    """
    def __init__(self, expression, message):
        self.expression = expression
        self.message = message


class GithubFileNotFound(Exception):
    """Exception raised when getting a 404 from github
    """
    def __init__(self, message):
        self.message = message



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2021 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Alvaro del Castillo <acs@bitergia.com>
#     Quan Zhou <quan@bitergia.com>
#


import json
import sys
import unittest

# Hack to make sure that tests import the right packages
# due to setuptools behaviour
sys.path.insert(0, '..')

from sgqlc.operation import Operation

from grimoire_elk.enriched.sortinghat_gelk import SortingHat

from sirmordred.config import Config
from sirmordred.task_identities import TaskIdentitiesMerge

from sortinghat.cli.client import SortingHatSchema


CONF_FILE = 'test.cfg'
REMOTE_IDENTITIES_FILE = 'data/remote_identities_sortinghat.json'
# REMOTE_IDENTITIES_FILE_URL = 'http://example.com/identities.json'
REMOTE_IDENTITIES_FILE_URL = 'https://github.com/fake/repo/identities.json'


def read_file(filename, mode='r'):
    with open(filename, mode) as f:
        content = f.read()
    return content


class TestTaskIdentitiesMerge(unittest.TestCase):
    """Task tests"""

    @staticmethod
    def get_organizations(task):
        args = {
            "page": 1,
            "page_size": 10
        }
        op = Operation(SortingHatSchema.Query)
        org = op.organizations(**args)
        org.entities().name()
        result = task.client.execute(op)
        organizations = result['data']['organizations']['entities']

        return organizations

    @staticmethod
    def delete_identity(task, args):
        op = Operation(SortingHatSchema.SortingHatMutation)
        identity = op.delete_identity(**args)
        identity.uuid()
        task.client.execute(op)

    @staticmethod
    def delete_organization(task, args):
        op = Operation(SortingHatSchema.SortingHatMutation)
        org = op.delete_organization(**args)
        org.organization.name()
        task.client.execute(op)

    @staticmethod
    def add_identity(task, args):
        op = Operation(SortingHatSchema.SortingHatMutation)
        identity = op.add_identity(**args)
        identity.uuid()
        task.client.execute(op)

    @staticmethod
    def add_organization(task, args):
        op = Operation(SortingHatSchema.SortingHatMutation)
        org = op.add_organization(**args)
        org.organization.name()
        task.client.execute(op)

    @staticmethod
    def add_domain(task, args):
        op = Operation(SortingHatSchema.SortingHatMutation)
        dom = op.add_domain(**args)
        dom.domain.domain()
        task.client.execute(op)

    def setUp(self):
        config = Config(CONF_FILE)
        task = TaskIdentitiesMerge(config)

        # Clean database
        # Remove identities
        entities = SortingHat.unique_identities(task.client)
        mks = [e['mk'] for e in entities]
        for i in mks:
            arg = {
                'uuid': i
            }
            self.delete_identity(task, arg)

        # Remove organization
        organizations = self.get_organizations(task)
        for org in organizations:
            self.delete_organization(task, org)

        data = json.loads(read_file("data/task-identities-data.json"))

        identities = data['identities']
        for identity in identities:
            self.add_identity(task, identity)

        organizations = data['organizations']
        for org in organizations:
            self.add_organization(task, {"name": org['organization']})
            self.add_domain(task, org)

    def test_initialization(self):
        """Test whether attributes are initializated"""

        config = Config(CONF_FILE)
        task = TaskIdentitiesMerge(config)

        self.assertEqual(task.config, config)

    def test_is_backend_task(self):
        config = Config(CONF_FILE)
        task = TaskIdentitiesMerge(config)

        self.assertFalse(task.is_backend_task())

    def test_execute(self):
        config = Config(CONF_FILE)
        task = TaskIdentitiesMerge(config)

        self.assertIsNone(task.execute())
        args = {
            'page': 1,
            'page_size': 10
        }
        op = Operation(SortingHatSchema.Query)
        op.individuals(**args)
        individual = op.individuals().entities()
        individual.profile().name()
        individual.enrollments().group().name()
        result = task.client.execute(op)
        entities = result['data']['individuals']['entities']

        enrolls = {}
        for e in entities:
            name = e['profile']['name']
            enroll = e['enrollments'][0]['group']['name']
            if name in enrolls:
                enrolls[name].append(enroll)
            else:
                enrolls[name] = [enroll]

        expected_enrolls = {
            "user2": ["org2"],
            "user3": ["org1"],
            "user4": ["org2", "org3"],
            "user6": ["org6"],
            "user7": ["org1"],
            "user8": ["org8"],
            "user9": ["org2"],
            "user10": ["org3"]
        }

        self.assertDictEqual(enrolls, expected_enrolls)
        self.assertEqual(len(entities), 9)


if __name__ == "__main__":
    # logging.basicConfig(level=logging.DEBUG, format='%(asctime)s %(message)s')
    unittest.main(buffer=True, warnings='ignore')



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Jose Javier Merchante <jjmerchante@bitergia.com>
#

import json
import logging
import sys
import unittest

import requests

# Hack to make sure that tests import the right packages
# due to setuptools behaviour
sys.path.insert(0, '..')


from grimoire_elk.enriched.sortinghat_gelk import SortingHat

from sirmordred.config import Config
from sirmordred.task_autorefresh import TaskAutorefresh
from sirmordred.task_collection import TaskRawDataCollection
from sirmordred.task_enrich import TaskEnrich
from sirmordred.task_projects import TaskProjects

from sortinghat.cli.client import SortingHatSchema

from sgqlc.operation import Operation


CONF_FILE = 'test.cfg'
GIT_BACKEND_SECTION = 'git'


logging.basicConfig(level=logging.INFO)


def read_file(filename, mode='r'):
    with open(filename, mode) as f:
        content = f.read()
    return content


class TestTaskAutorefresh(unittest.TestCase):
    """Test TaskAutorefresh"""

    @staticmethod
    def get_organizations(client):
        args = {
            "page": 1,
            "page_size": 10
        }
        op = Operation(SortingHatSchema.Query)
        org = op.organizations(**args)
        org.entities().name()
        result = client.execute(op)
        organizations = result['data']['organizations']['entities']

        return organizations

    @staticmethod
    def get_individuals(client):
        args = {
            "page": 1,
            "page_size": 1000
        }
        op = Operation(SortingHatSchema.Query)
        individual = op.individuals(**args)
        individual.entities().mk()
        individual.entities().profile().name()
        result = client.execute(op)

        return result['data']['individuals']['entities']

    @staticmethod
    def delete_identity(client, args):
        op = Operation(SortingHatSchema.SortingHatMutation)
        identity = op.delete_identity(**args)
        identity.uuid()
        client.execute(op)

    @staticmethod
    def delete_organization(client, args):
        op = Operation(SortingHatSchema.SortingHatMutation)
        org = op.delete_organization(**args)
        org.organization.name()
        client.execute(op)

    @staticmethod
    def merge_individuals(client, args):
        op = Operation(SortingHatSchema.SortingHatMutation)
        merge = op.merge(**args)
        merge.individual().mk()
        client.execute(op)

    @staticmethod
    def add_identity(client, args):
        op = Operation(SortingHatSchema.SortingHatMutation)
        identity = op.add_identity(**args)
        identity.uuid()
        client.execute(op)

    @staticmethod
    def add_organization(client, args):
        op = Operation(SortingHatSchema.SortingHatMutation)
        org = op.add_organization(**args)
        org.organization.name()
        client.execute(op)

    @staticmethod
    def add_domain(client, args):
        op = Operation(SortingHatSchema.SortingHatMutation)
        dom = op.add_domain(**args)
        dom.domain.domain()
        client.execute(op)

    def setUp(self):
        config = Config(CONF_FILE)
        task = TaskAutorefresh(config)

        # Clean database
        entities = SortingHat.unique_identities(task.client)
        mks = [e['mk'] for e in entities]
        for i in mks:
            arg = {
                'uuid': i
            }
            self.delete_identity(task.client, arg)

        organizations = self.get_organizations(task.client)
        for org in organizations:
            self.delete_organization(task.client, org)

        # Add identities and organizations
        data = json.loads(read_file("data/task-identities-data.json"))

        identities = data['identities']
        for identity in identities:
            self.add_identity(task.client, identity)

        organizations = data['organizations']
        for org in organizations:
            self.add_organization(task.client, {"name": org['organization']})
            self.add_domain(task.client, org)

    def test_initialization(self):
        """Test whether attributes are initialized"""

        config = Config(CONF_FILE)
        task = TaskAutorefresh(config)

        self.assertEqual(task.config, config)

    def test_is_backend_task(self):
        """Test whether the Task is not a backend task"""

        config = Config(CONF_FILE)
        task = TaskAutorefresh(config)

        self.assertFalse(task.is_backend_task())

    def test_execute(self):
        """Test whether the Task could be run"""

        # Create a raw and enriched indexes
        config = Config(CONF_FILE)

        TaskProjects(config).execute()
        backend_section = GIT_BACKEND_SECTION

        task_collection = TaskRawDataCollection(config, backend_section=backend_section)
        task_collection.execute()

        task_enrich = TaskEnrich(config, backend_section=backend_section)
        task_enrich.execute()

        task_autorefresh = TaskAutorefresh(config)
        task_autorefresh.config.set_param('es_enrichment', 'autorefresh', True)
        # This does nothing because it uses now as from_date:
        task_autorefresh.execute()

        # Merge all the identities into user1
        individuals = self.get_individuals(task_autorefresh.client)
        to_uuid = '72f6fc79632080fc17dc8f83c0ddd5ff5b5e5005'
        from_uuids = [ind['mk'] for ind in individuals if ind['mk'] != to_uuid]
        arg = {'from_uuids': from_uuids, 'to_uuid': to_uuid}
        self.merge_individuals(task_autorefresh.client, arg)

        self.assertIsNone(task_autorefresh.execute())

        # Check that the autorefresh went well
        cfg = config.get_conf()
        es_enrichment = cfg['es_enrichment']['url']
        enrich_index = es_enrichment + "/" + cfg[GIT_BACKEND_SECTION]['enriched_index']

        r = requests.get(enrich_index + "/_search", verify=False)
        for hit in r.json()['hits']['hits']:
            self.assertEqual(hit['_source']['author_uuid'], to_uuid)


if __name__ == "__main__":
    unittest.main(buffer=True, warnings='ignore')



# -*- coding: utf-8 -*-
#
# Copyright (C) 2023 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Jose Javier Merchante <jjmerchante@bitergia.com>
#

import logging
import ssl

from datetime import datetime

from elasticsearch import Elasticsearch
from elasticsearch.connection import create_ssl_context

from grimoire_elk.elk import refresh_identities
from grimoire_elk.enriched.git import GitEnrich
from grimoire_elk.utils import get_elastic, get_connector_from_name
from grimoire_elk.enriched.sortinghat_gelk import SortingHat

from sirmordred.config import Config
from sirmordred.task import Task
from sirmordred.task_projects import TaskProjects


logger = logging.getLogger(__name__)


class TaskAutorefresh(Task):
    """Refresh the last modified identities for all the backends."""

    def __init__(self, config):
        super().__init__(config)

        self.last_autorefresh_backend = {}

    def is_backend_task(self):
        return False

    def _get_backend_sections(self):
        """Get all the backends' sections enabled"""

        backends = []
        projects = TaskProjects.get_projects()

        for pro in projects:
            for sect in projects[pro].keys():
                for backend_section in Config.get_backend_sections():
                    if sect.startswith(backend_section) and sect in self.conf:
                        backends.append(sect)

        # Remove duplicates
        backends = list(set(backends))
        return backends

    def __autorefresh(self, enrich_backend, backend_section, after):
        """Refresh identities for a specific backend or study after a specific date"""

        logger.info(f"[{backend_section}] Refreshing identities from {after}")

        field_id = enrich_backend.get_field_unique_id()
        author_fields = ["author_uuid"]
        try:
            meta_fields = enrich_backend.meta_fields
            author_fields += meta_fields
        except AttributeError:
            pass

        total_indiv = 0
        total_items = 0
        time_start = datetime.now()
        for individuals in SortingHat.search_last_modified_identities(self.client, after):
            eitems = refresh_identities(enrich_backend, author_fields=author_fields, individuals=individuals)
            total_items += enrich_backend.elastic.bulk_upload(eitems, field_id)
            total_indiv += len(individuals)
            logger.debug(f"[{backend_section}] Individuals/items refreshed: {total_indiv}/{total_items}")

        spent_time = str(datetime.now() - time_start).split('.')[0]
        logger.info(f'[{backend_section}] Refreshed {total_indiv} individuals and {total_items} items in {spent_time}')

    def __autorefresh_areas_of_code(self, after):
        """Execute autorefresh for areas of code study if configured"""

        if 'git' not in self.conf or \
           'studies' not in self.conf['git'] or \
           'enrich_areas_of_code:git' not in self.conf['git']['studies']:
            logger.info("Not doing autorefresh for studies, Areas of Code study is not active.")
            return

        aoc_index = self.conf['enrich_areas_of_code:git'].get('out_index', GitEnrich.GIT_AOC_ENRICHED)
        if not aoc_index:
            aoc_index = GitEnrich.GIT_AOC_ENRICHED

        logger.debug(f"Autorefresh for Areas of Code study index: {aoc_index}")

        ssl_context = create_ssl_context()
        ssl_context.check_hostname = False
        ssl_context.verify_mode = ssl.CERT_NONE
        es = Elasticsearch(hosts=[self.conf['es_enrichment']['url']],
                           timeout=100, retry_on_timeout=True, ssl_context=ssl_context)

        if not es.indices.exists(index=aoc_index):
            logger.debug("Not doing autorefresh, index doesn't exist for Areas of Code study")
            return

        logger.debug("Doing autorefresh for Areas of Code study")

        # Create a GitEnrich backend tweaked to work with AOC index
        json_projects_map = self.conf['projects']['projects_file']
        aoc_backend = GitEnrich(db_sortinghat=self.db_sh, json_projects_map=json_projects_map,
                                db_user=self.db_user, db_password=self.db_password, db_host=self.db_host,
                                db_port=self.db_port, db_path=self.db_path, db_ssl=self.db_ssl,
                                db_verify_ssl=self.db_verify_ssl, db_tenant=self.db_tenant)
        aoc_backend.mapping = None
        aoc_backend.roles = ['author']
        elastic_enrich = get_elastic(url=self.conf['es_enrichment']['url'], es_index=aoc_index,
                                     clean=False, backend=aoc_backend)
        aoc_backend.set_elastic(elastic_enrich)
        if self.db_unaffiliate_group:
            aoc_backend.unaffiliated_group = self.db_unaffiliate_group

        self.__autorefresh(aoc_backend, 'git:aoc', after)

    def __get_enrich_backend(self, backend):
        connector = get_connector_from_name(backend)

        json_projects_map = self.conf['projects']['projects_file']
        enrich_backend = connector[2](db_sortinghat=self.db_sh, json_projects_map=json_projects_map,
                                      db_user=self.db_user, db_password=self.db_password, db_host=self.db_host,
                                      db_port=self.db_port, db_path=self.db_path, db_ssl=self.db_ssl,
                                      db_verify_ssl=self.db_verify_ssl, db_tenant=self.db_tenant)
        elastic_enrich = get_elastic(url=self.conf['es_enrichment']['url'],
                                     es_index=self.conf[backend]['enriched_index'],
                                     clean=False, backend=enrich_backend)
        enrich_backend.set_elastic(elastic_enrich)

        if 'pair-programming' in self.conf[backend]:
            enrich_backend.pair_programming = self.conf[backend]['pair-programming']

        if self.db_unaffiliate_group:
            enrich_backend.unaffiliated_group = self.db_unaffiliate_group

        return enrich_backend

    def execute(self):
        """Run autorefresh for all the backends"""

        autorefresh = self.conf['es_enrichment']['autorefresh']
        if not autorefresh or not self.client:
            logger.info('Periodic autorefresh not active')
            return

        backends = self._get_backend_sections()
        for backend_section in backends:
            after = self.last_autorefresh_backend.get(backend_section, datetime.utcnow())
            self.last_autorefresh_backend[backend_section] = datetime.utcnow()

            logger.info(f'[{backend_section}] Periodic autorefresh start')
            enrich_backend = self.__get_enrich_backend(backend_section)
            self.__autorefresh(enrich_backend, backend_section, after)
            logger.info(f'[{backend_section}] Periodic autorefresh end')

        after = self.last_autorefresh_backend.get('git:aoc', datetime.utcnow())
        self.last_autorefresh_backend['git:aoc'] = datetime.utcnow()

        logger.info('[git:aoc] Periodic autorefresh for studies starts')
        self.__autorefresh_areas_of_code(after)
        logger.info('[git:aoc] Periodic autorefresh for studies ends')
